//
//  main.c
//
//  Program main / I2C_LED7SEG_x8 0.1a
//  Copyright (C) 2010-2011 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I2C and 7segment LED x 8 dynamic display driver.
//
//	Setting
//		ATtiny
//			Device	: ATtiny 2313
//			IDE		: AVR Studio 4.18.700  
//			Compiler: WinAVR 20100110(GCC 4.3.3)
//
//	Usage
//		Pin assign:
//			PA0 - Segment f
//			PA1 - Segment g
//			PA2 - Segment dp
//			PB0 - Segment a
//			PB1 - Segment b
//			PB2 - Segment c
//			PB3 - Segment d
//			PB4 - Segment e
//			PB5 - I2C SDA
//			PB6 - Common 0(column0)
//			PB7 - I2C SCL
//			PD0 - Common 7(column7)
//			PD1 - Common 6(column6)
//			PD2 - Common 5(column5)
//			PD3 - Common 4(column4)
//			PD4 - Common 3(column3)
//			PD5 - Common 2(column2)
//			PD6 - Common 1(column1)
//
//		I2C data stream:
//
//			Put data flow
//			            SAD   R/W     CMD-PRM        DATA                       DATA
//			       +-+-------+---+   +--------+   +--------+   +-- ... --+   +--------+   +-+
//			Master |S|0000111| 0 |   |00000000|   |00000000|   |         |   |00000000|   |P|
//			       +-+-------+---+---+--------+---+--------+---+-- ... --+---+--------+---+-+
//			Slave                | 0 |        | 0 |        | 0 |         | 0 |        | 0 |
//			 (my)                +---+        +---+        +---+         +---+        +---+
//			                    NAK/ACK      NAK/ACK      NAK/ACK       NAK/ACK      NAK/ACK
//
//			Get data flow
//			        S   SAD   R/W            NAK/ACK       NAK/ACK      NAK/ACK
//			       +-+-------+---+            +---+         +---+        +---+-+
//			Master |S|0000111| 1 |            | 0 |         | 0 |        | 1 |P|
//			       +-+-------+---+---+--------+---+-- ... --+---+--------+---+-+
//			Slave                | 0 |00000000|   |         |   |00000000|
//			 (my)                +---+--------+   +-- ... --+   +--------+
//			                    NAK/ACK DATA                       DATA
//
//				S      : Start condition
//				SAD    : Slave address [7bits]
//				R/W    : Read(1) or Write(0) [1bit]
//				NAK/ACK: Acknowledge(0) or Not(1) [1bit]
//				P      : Stop condition
//				CMD-PRM: Command code[4bits] and Parameter[4bits]
//					CMD  PRM
//					---- ----
//					0000 aaaa  Set get/put location address.
//					0001 dddd  Set dimmer level.
//					     0000     100%
//					     0001     50%
//					     0010     25%
//					     0011     12%
//					     0100     6%
//					     0101     3%
//			    	0010 ssss  Scan start/stop.
//					     0000     Scan stop
//					     0001     Scan start
//				DATA   : Display bit image data[8bits]
//					bits  segment
//			        ----  -------
//					b0    a            aaa     0=off
//					b1    b           f   b    1=on
//					b2    c           f   b
//					b3    d            ggg
//					b4    e           e   c
//					b5    f           e   c
//					b6    g            ddd
//					b7    dp               dp
//
//					location  display
//			        --------  -------
//					0000      Column0
//					0001      Column1
//					0010      Column2
//					      :
//					0111      Column7
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2010.06.02 v0.0  The first cording
//	2010.06.20 v0.1  Fixed
//	2011.02.28 v0.1a Column pin conversely modification
//


#include "avr/io.h"
#include "avr/interrupt.h"
#include "avr/sleep.h"


//
//	Parameter
//
#define I2C_S_SAD			0x09		// Slave address
#define DRIVE_MODE_ROW_ACTIVE_LOW		// Segment line (active high is comment out)
#define DRIVE_MODE_COLUMN_ACTIVE_LOW	// Common line (active high is comment out)


//
//	Global area
//
unsigned char 	dreg[] = {				// Display data register
//    .gfedcba
	0b00111111,		// Col0
	0b00000110,		// Col1
	0b01011011,		// Col2
	0b01001111,		// Col3
	0b01100110,		// Col4
	0b01101101,		// Col5
	0b01111101,		// Col6
	0b10000111,		// Col7
	0b00000000,		// Col0-back
	0b00000000,		// Col1-back
	0b00000000,		// Col2-back
	0b00000000,		// Col3-back
	0b00000000,		// Col4-back
	0b00000000,		// Col5-back
	0b00000000,		// Col6-back
	0b00000000		// Col7-back
};
unsigned char 	ddim = 0x00;			// Display dimer


//
//	Subroutines
//

// LED display refresh					(Row:PA0-2,PB0-4  Col:PD0-6,PB6)
void led_refresh(void) {
	static unsigned char	ctr  = 0;	// Display refresh counter [0..255]
	static unsigned char	ctr2 = 0;	// Low brighte counter [0..255]
	static unsigned char	*ptr = dreg;
	unsigned char			d;			// bit image
	unsigned char			a;			// port a data
	unsigned char			b;			// port b data

	// Build row port data
	if ((ddim != 0) && (ctr & ddim)) {
		d = 0;
	} else {
		d = *ptr;
		if ((ctr2 & 0x7) == 0) {		// 1/8 cycle
			d |= *(ptr+8);				// Low bright segment
		}
	}
#ifdef DRIVE_MODE_ROW_ACTIVE_LOW
	d ^= 0xff;
#endif
	a = (d >> 5);
	b = (d & 0b00011111);

	// All off column
#ifdef DRIVE_MODE_COLUMN_ACTIVE_LOW
	PORTD = 0xff;
	PORTB |= _BV(PB6);
#else
	PORTD = 0x00;
	PORTB &= ~_BV(PB6);
#endif

	// Change
	PORTA = a;
	PORTB = (PORTB & ~(_BV(PB0) | _BV(PB1) | _BV(PB2) | _BV(PB3) | _BV(PB4))) | b;

	// Taget column On
#ifdef DRIVE_MODE_COLUMN_ACTIVE_LOW
	switch(ctr & 0x7) {
//	case 0:	PORTD &= ~_BV(PD0);	break;	// DEL 2011.02.28
//	case 1:	PORTD &= ~_BV(PD1);	break;	// DEL 2011.02.28
//	case 2:	PORTD &= ~_BV(PD2);	break;	// DEL 2011.02.28
//	case 3:	PORTD &= ~_BV(PD3);	break;	// DEL 2011.02.28
//	case 4:	PORTD &= ~_BV(PD4);	break;	// DEL 2011.02.28
//	case 5:	PORTD &= ~_BV(PD5);	break;	// DEL 2011.02.28
//	case 6:	PORTD &= ~_BV(PD6);	break;	// DEL 2011.02.28
//	default:PORTB &= ~_BV(PB6);	break;	// DEL 2011.02.28
	case 0: PORTB &= ~_BV(PB6);	break;	// ADD 2011.02.28
	case 1:	PORTD &= ~_BV(PD6);	break;	// ADD 2011.02.28
	case 2:	PORTD &= ~_BV(PD5);	break;	// ADD 2011.02.28
	case 3:	PORTD &= ~_BV(PD4);	break;	// ADD 2011.02.28
	case 4:	PORTD &= ~_BV(PD3);	break;	// ADD 2011.02.28
	case 5:	PORTD &= ~_BV(PD2);	break;	// ADD 2011.02.28
	case 6:	PORTD &= ~_BV(PD1);	break;	// ADD 2011.02.28
	default:PORTD &= ~_BV(PD0);	break;	// ADD 2011.02.28
	}
#else
	switch(ctr & 0x7) {
//	case 0:	PORTD |= _BV(PD0);	break;	// DEL 2011.02.28
//	case 1:	PORTD |= _BV(PD1);	break;	// DEL 2011.02.28
//	case 2:	PORTD |= _BV(PD2);	break;	// DEL 2011.02.28
//	case 3:	PORTD |= _BV(PD3);	break;	// DEL 2011.02.28
//	case 4:	PORTD |= _BV(PD4);	break;	// DEL 2011.02.28
//	case 5:	PORTD |= _BV(PD5);	break;	// DEL 2011.02.28
//	case 6:	PORTD |= _BV(PD6);	break;	// DEL 2011.02.28
//	default:PORTB |= _BV(PB6);	break;	// DEL 2011.02.28
	case 0:	PORTB |= _BV(PB6);	break;	// ADD 2011.02.28
	case 1:	PORTD |= _BV(PD6);	break;	// ADD 2011.02.28
	case 2:	PORTD |= _BV(PD5);	break;	// ADD 2011.02.28
	case 3:	PORTD |= _BV(PD4);	break;	// ADD 2011.02.28
	case 4:	PORTD |= _BV(PD3);	break;	// ADD 2011.02.28
	case 5:	PORTD |= _BV(PD2);	break;	// ADD 2011.02.28
	case 6:	PORTD |= _BV(PD1);	break;	// ADD 2011.02.28
	default:PORTD |= _BV(PD0);	break;	// ADD 2011.02.28
	}
#endif

	// Next
	ctr++;
	ptr++;
	if ((ctr & 0x7) == 0) {
		ptr = dreg;
		ctr2++;
	}

	return;
}

// I2C Slave
void i2c_slave_task(void) {

#define I2C_S_STATE_IDLE	0
#define I2C_S_STATE_SADRCV	1
#define I2C_S_STATE_ACKSND	2
#define I2C_S_STATE_DATRCV	3
#define I2C_S_STATE_DATSND	4
#define I2C_S_STATE_ACKRCV	5
#define I2C_S_STATE_SIZE	6

#define I2C_S_EVENT_STARTC	0
#define I2C_S_EVENT_STOPC	1
#define I2C_S_EVENT_SNDRCV0	2			// SAD-unmatch  W-Mode  ACK
#define I2C_S_EVENT_SNDRCV1	3			// SAD-match    R-Mode  NAK
#define I2C_S_EVENT_COLS	4			// Collision
//#define I2C_S_EVENT_TMOUT	5
#define I2C_S_EVENT_SIZE	5
#define I2C_S_EVENT_NONE	99


#define I2C_S_ACTION_RCV1	1
#define I2C_S_ACTION_RCV8	2
#define I2C_S_ACTION_SADACK	3
#define I2C_S_ACTION_DATACK	4
#define I2C_S_ACTION_DATSND	5
#define I2C_S_ACTION_NONE	99

	// Communication flow decision table
	const struct {
		unsigned char	action;
		unsigned char	next_state;
	} tbl[I2C_S_STATE_SIZE][I2C_S_EVENT_SIZE] = {
		/* STATE                    EVENT_STARTC                            EVENT_STOPC                          EVENT_SNDRCV0                             EVENT_SNDRCV1                              EVENT_COLS            */
		/* IDLE */   { {I2C_S_ACTION_RCV8,I2C_S_STATE_SADRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE}, {I2C_S_ACTION_NONE,  I2C_S_STATE_IDLE},   {I2C_S_ACTION_NONE,  I2C_S_STATE_IDLE},   {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE} },
		/* SADRCV */ { {I2C_S_ACTION_RCV8,I2C_S_STATE_SADRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE}, {I2C_S_ACTION_NONE,  I2C_S_STATE_IDLE},   {I2C_S_ACTION_SADACK,I2C_S_STATE_ACKSND}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE} },
		/* ACKSND */ { {I2C_S_ACTION_RCV8,I2C_S_STATE_SADRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE}, {I2C_S_ACTION_RCV8,  I2C_S_STATE_DATRCV}, {I2C_S_ACTION_DATSND,I2C_S_STATE_DATSND}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE} },
		/* DATRCV */ { {I2C_S_ACTION_RCV8,I2C_S_STATE_SADRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE}, {I2C_S_ACTION_DATACK,I2C_S_STATE_ACKSND}, {I2C_S_ACTION_DATACK,I2C_S_STATE_ACKSND}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE} },
		/* DATSND */ { {I2C_S_ACTION_RCV8,I2C_S_STATE_SADRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE}, {I2C_S_ACTION_RCV1,  I2C_S_STATE_ACKRCV}, {I2C_S_ACTION_RCV1,  I2C_S_STATE_ACKRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE} },
		/* ACKRCV */ { {I2C_S_ACTION_RCV8,I2C_S_STATE_SADRCV}, {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE}, {I2C_S_ACTION_DATSND,I2C_S_STATE_DATSND}, {I2C_S_ACTION_NONE,  I2C_S_STATE_IDLE},   {I2C_S_ACTION_NONE,I2C_S_STATE_IDLE} }
	};
	static unsigned char	dp = 0;		// Display data I/O current pointer
	static unsigned char	st = I2C_S_STATE_IDLE;
	static unsigned char	rw = 0;		// R/W
	static unsigned char	cf = 1;		// Command flag
	unsigned char			e;			// Event code
	unsigned char			d;			// USI Data register copy


	// Catch event
	d = USIDR;
	if (bit_is_set(USISR, USISIF)) {
		e = I2C_S_EVENT_STARTC;
	} else if (bit_is_set(USISR, USIOIF)) {
		if (st == I2C_S_STATE_SADRCV) {
			e = ((d >> 1) == I2C_S_SAD) ? I2C_S_EVENT_SNDRCV1 : I2C_S_EVENT_SNDRCV0;
		} else if (st == I2C_S_STATE_ACKSND) {
			e = rw ? I2C_S_EVENT_SNDRCV1 : I2C_S_EVENT_SNDRCV0;
		} else {
			e = d ? I2C_S_EVENT_SNDRCV1 : I2C_S_EVENT_SNDRCV0;
		}
	} else if (bit_is_set(USISR, USIPF)) {
		e = I2C_S_EVENT_STOPC;
	} else if (bit_is_set(USISR, USIDC)) {
		e = I2C_S_EVENT_COLS;
	} else {
		e = I2C_S_EVENT_NONE;
	}

	// Action
	if (e != I2C_S_EVENT_NONE) {

		switch(tbl[st][e].action){

		case I2C_S_ACTION_RCV1:				// Kick recieve 1 bit
			DDRB &= ~_BV(DDB5);				// SDA input
			USISR =							// USI Status Register
				_BV(USISIF)  |				// Start Condition Interrupt Flag
				_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
				_BV(USIPF)   |				// Stop Condition Flag
				_BV(USIDC)   |				// Data Output Collision
				_BV(USICNT3) |				// Counter Value
				_BV(USICNT2) |				//  
				_BV(USICNT1) |				//    0x0e = 2 rise+fall edge SCK
			//	_BV(USICNT0) |				// 
				0;
			break;

		case I2C_S_ACTION_RCV8:				// Kick recieve 8 bits
			DDRB &= ~_BV(DDB5);				// SDA input
			USISR =							// USI Status Register
				_BV(USISIF)  |				// Start Condition Interrupt Flag
				_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
				_BV(USIPF)   |				// Stop Condition Flag
				_BV(USIDC)   |				// Data Output Collision
			//	_BV(USICNT3) |				// Counter Value
			//	_BV(USICNT2) |				//  
			//	_BV(USICNT1) |				//    0 = 16 double edge SCK
			//	_BV(USICNT0) |				// 
				0;
			break;

		case I2C_S_ACTION_SADACK:			// SAD recieved after ACK send
			rw    = d & _BV(0);				// Set R/W(b0) flag
			cf    = 1;						// Set command byte

			DDRB |= _BV(DDB5);				// SDA output
			USIDR = 0;						// ACK data
			USISR =							// USI Status Register
				_BV(USISIF)  |				// Start Condition Interrupt Flag
				_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
				_BV(USIPF)   |				// Stop Condition Flag
				_BV(USIDC)   |				// Data Output Collision
				_BV(USICNT3) |				// Counter Value
				_BV(USICNT2) |				// 
				_BV(USICNT1) |				//   0x0e = 2 rise+fall edge SCK
			//	_BV(USICNT0) |				// 
				0;
			break;

		case I2C_S_ACTION_DATACK:			// DATA recieved after ACK send
			if (cf) {						// Command byte
				unsigned char	cmd;		//
				unsigned char	prm;		//

				cf  = 0;					//
				cmd = d & 0b11110000;		//
				prm = d & 0b00001111;		//
				switch(cmd) {				//
				case 0x00:					// Set address command
					dp = prm;				//
					break;					//
				case 0x10:					// Set dimmer command
					switch(prm){
					case 0:	ddim = 0;		break;
					case 1:	ddim = 0x08;	break;
					case 2:	ddim = 0x18;	break;
					case 3:	ddim = 0x38;	break;
					case 4:	ddim = 0x78;	break;
					case 5:	ddim = 0xf8;	break;
					default:				break;
					}
					break;					//
				case 0x20:					// Scan start/stop command
					if (prm) {				//
						TIMSK |= _BV(OCIE0A);	// Interrupt Enable
					} else {				//
						TIMSK &= ~_BV(OCIE0A);	// Interrupt Disable
					}						//
					break;					//
				default:					//
					break;					//
				}							//
			} else {						// Data bytes
				dreg[dp++] = d;				// Set data
				if (dp >= sizeof(dreg)) {	//
					dp = 0;					//
				}							//
			}								//
			DDRB |= _BV(DDB5);				// SDA output
			USIDR = 0;						// ACK data
			USISR =							// USI Status Register
				_BV(USISIF)  |				// Start Condition Interrupt Flag
				_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
				_BV(USIPF)   |				// Stop Condition Flag
				_BV(USIDC)   |				// Data Output Collision
				_BV(USICNT3) |				// Counter Value
				_BV(USICNT2) |				// 
				_BV(USICNT1) |				//   0x0e = 2 rise+fall edge SCK
			//	_BV(USICNT0) |				// 
				0;
			break;

		case I2C_S_ACTION_DATSND:
			DDRB |= _BV(DDB5);				// SDA output
			USIDR = dreg[dp++];				// Data set
			if (dp >= sizeof(dreg)) {
				dp = 0;
			}
			USISR =							// USI Status Register
				_BV(USISIF)  |				// Start Condition Interrupt Flag
				_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
				_BV(USIPF)   |				// Stop Condition Flag
				_BV(USIDC)   |				// Data Output Collision
			//	_BV(USICNT3) |				// Counter Value
			//	_BV(USICNT2) |				// 
			//	_BV(USICNT1) |				//   0x00 = 16 rise+fall edge SCK
			//	_BV(USICNT0) |				// 
				0;
			break;

		default:
			USISR =							// USI Status Register  <- *RESET*
				_BV(USISIF)  |				// Start Condition Interrupt Flag
				_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
				_BV(USIPF)   |				// Stop Condition Flag
				_BV(USIDC)   |				// Data Output Collision
			//	_BV(USICNT3) |				// Counter Value
			//	_BV(USICNT2) |				//
			//	_BV(USICNT1) |				//
			//	_BV(USICNT0) |				//
				0;
			break;
		}

		// Next state
		st = tbl[st][e].next_state;
	}

	return;
}


//
//	ISR
//

// Refresh display
ISR(TIMER0_COMPA_vect) {
	led_refresh();
	return;
}

// I2C start condition
ISR(USI_START_vect) {
	i2c_slave_task();
	return;
}

// I2C counter overflow
ISR_ALIAS(USI_OVERFLOW_vect, USI_START_vect);


//
//	M a i n
//
main(){
	//
	// Config
	//

	// Main clock
	CLKPR = _BV(CLKPCE);				// Change sequence
	CLKPR = 							// Clock Prescale Register
		//	_BV(CLKPS3) |				// Clock Prescaler Select Bits  0 0 0 0 0 0 0 0 1
		//	_BV(CLKPS2) |				//                              0 0 0 0 1 1 1 1 0
			_BV(CLKPS1) |				//                              0 0 1 1 0 0 1 1 0
			_BV(CLKPS0) |				//                              0 1 0 1 0 1 0 1 0
										//                              +-|-|-|-|-|-|-|-|-- 1/1
										//                                +-|-|-|-|-|-|-|-- 1/2
										//                                  +-|-|-|-|-|-|-- 1/4
										//                                    +-|-|-|-|-|-- 1/8
										//                                      +-|-|-|-|-- 1/16
										//                                        +-|-|-|-- 1/32
										//                                          +-|-|-- 1/64
										//                                            +-|-- 1/128
										//                                              +-- 1/256
			0;							//

	// LED port
	DDRA  |= (_BV(DDRA0)| _BV(DDRA1)| _BV(DDRA2));
	DDRB  |= (_BV(DDB0) | _BV(DDB1) | _BV(DDB2) | _BV(DDB3) | _BV(DDB4)             | _BV(DDB6));
	DDRD  |= (_BV(DDD0) | _BV(DDD1) | _BV(DDD2) | _BV(DDD3) | _BV(DDD4) | _BV(DDD5) | _BV(DDD6));

#ifdef DRIVE_MODE_ROW_ACTIVE_LOW
	PORTA |=  (_BV(PA0) | _BV(PA1) | _BV(PA2));
	PORTB |=  (_BV(PB0) | _BV(PB1) | _BV(PB2) | _BV(PB3) | _BV(PB4));
#else
	PORTA &= ~(_BV(PA0) | _BV(PA1) | _BV(PA2));
	PORTB &= ~(_BV(PB0) | _BV(PB1) | _BV(PB2) | _BV(PB3) | _BV(PB4));
#endif
#ifdef DRIVE_MODE_COLUMN_ACTIVE_LOW
	PORTD |=  (_BV(PD0) | _BV(PD1) | _BV(PD2) | _BV(PD3) | _BV(PD4) | _BV(PD5) | _BV(PD6));
	PORTB |=  _BV(PB6);
#else
	PORTD &= ~(_BV(PD0) | _BV(PD1) | _BV(PD2) | _BV(PD3) | _BV(PD4) | _BV(PD5) | _BV(PD6));
	PORTB &= ~_BV(PB6);
#endif

	// Timer/Counter0
	TCCR0A = 							// Timer/Counter0 Control Register A
		//	_BV(COM0A1) |				// Compare Match Output A Mode  0  0  1  1
		//	_BV(COM0A0) |				//                              0  1  0  1   non-PWM Mode                               Fast PWM Mode                                        Phase Correct PWM Mode
										//                              +--|--|--|-- Normal port operation, OC0A disconnected   Normal port operation, OC0A disconnected.            Normal port operation, OC0A disconnected
										//                                 +--|--|-- Toggle OC0A on Compare Match               WGM02=0: Normal Port Operation, OC0A Disconnected.   WGM02=0: Normal Port Operation, OC0A Disconnected 
										//                                    |  |                                              WGM02=1: Toggle OC0A on Compare Match.               WGM02=1: Toggle OC0A on Compare Match
										//                                    +--|-- Clear OC0A on Compare Match                Clear OC0A on Compare Match, set OC0A at TOP         Clear OC0A on Compare Match when up-counting. Set OC0A on Compare Match when down-counting
										//                                       +-- Set OC0A on Compare Match                  Set OC0A on Compare Match, clear OC0A at TOP         Set OC0A on Compare Match when up-counting. Clear OC0A on Compare Match when down-counting
		//	_BV(COM0B1) |				// Compare Match Output B Mode  0  0  1  1
		//	_BV(COM0B0) |				//                              0  1  0  1   non-PWM Mode                               Fast PWM Mode                                        Phase Correct PWM Mode
										//                              +--|--|--|-- Normal port operation, OC0B disconnected   Normal port operation, OC0B disconnected             Normal port operation, OCR0B disconnected
										//                                 +--|--|-- Toggle OC0B on Compare Match               Reserved                                             Reserved
										//                                    +--|-- Clear OC0B on Compare Match                Clear OC0B on Compare Match, set OC0B at TOP         Clear ORC0B on Compare Match when up-counting. Set OCR0B on Compare Match when down-counting
										//                                       +-- Set OC0B on Compare Match                  Set OC0B on Compare Match, clear OC0B at TOP         Set OCR0B on Compare Match when up-counting. Clear OCR0B on Compare Match when down-counting
		//	   (WGM02)->TCCR0B			// Waveform Generation Mode  0  0  0  0  1  1  1  1
			_BV(WGM01)  |				//                           0  0  1  1  0  0  1  1                             Update of   TOV Flag 
		//	_BV(WGM00)  |				//                           0  1  0  1  0  1  0  1   Mode               TOP    OCRx at     Set on
										//                           +--|--|--|--|--|--|--|-- Normal             0xFF   Immediate   MAX
										//                              +--|--|--|--|--|--|-- PWM, Phase Correct 0xFF   TOP         BOTTOM
										//                                 +--|--|--|--|--|-- CTC                OCR0A  Immediate   MAX
										//                                    +--|--|--|--|-- Fast PWM           0xFF   TOP         MAX
										//                                       +--|--|--|-- Reserved            -      -           -
										//                                          +--|--|-- PWM, Phase Correct OCR0A  TOP         BOTTOM
										//                                             +--|-- Reserved            -      -           -
										//                                                +-- Fast PWM           OCR0A  TOP         TOP
			0;							//
	TCCR0B = 							// Timer/Counter0 Control Register B
		//	_BV(FOC0A)  |				// Force Output Compare A
		//	_BV(FOC0B)  |				// Force Output Compare B
		//	_BV(WGM02)  |   			// Waveform Generation Mode -> See TCCR0A
		//	_BV(CS02)   |				// Clock Select   0  0  0  0  1  1  1  1
		//	_BV(CS01)   |				//                0  0  1  1  0  0  1  1
			_BV(CS00)   |				//                0  1  0  1  0  1  0  1
										//                +--|--|--|--|--|--|--|-- No clock source(Timer/Counter stopped)
										//                   +--|--|--|--|--|--|-- clkI/O/(No prescaling)
										//                      +--|--|--|--|--|-- clk I/O/8 (From prescaler)
										//                         +--|--|--|--|-- clkI/O/64 (From prescaler)
										//                            +--|--|--|-- clk I/O/256 (From prescaler)
										//                               +--|--|-- clk I/O/1024 (From prescaler)
										//                                  +--|-- External clock source on T0 pin. Clock on falling edge.
										//                                     +-- External clock source on T0 pin. Clock on rising edge.
			0;							//
	TCNT0 = 0;							// Timer/Counter Register
	OCR0A = 223;						// Output Compare Register A  <- 8MHz/CLKPS(8)/CS(1)/223=4.48kHz (LED refresh is 560Hz)
	OCR0B = 0;							// Output Compare Register B  <- Unuse

	// Timer/Counters interrupt
	TIMSK = 							// Timer/Counter Interrupt Mask Register
		//	_BV(TOIE1)  |				// Timer/Counter1 Overflow Interrupt Enable
		//	_BV(OCIE1A) |				// Timer/Counter1 Output Compare A Match Interrupt Enable
		//	_BV(OCIE1B) |				// Timer/Counter1 Output Compare B Match Interrupt Enable
		//	_BV(ICIE1)  |				// Timer/Counter1 Input Capture Interrupt Enable
		//	_BV(OCIE0B) |				// Timer/Counter0 Output Compare Match B Interrupt Enable
		//	_BV(TOIE0)  |				// Timer/Counter0 Overflow Interrupt Enable
			_BV(OCIE0A) |				// Timer/Counter0 Output Compare Match A Interrupt Enable
			0;							//
	TIFR = 								// Timer/Counter Interrupt Flag Register
		//	_BV(TOV1)   |				// Timer/Counter1 Overflow Flag
		//	_BV(OCF1A)  |				// Timer/Counter1 Output Compare A Match Flag
		//	_BV(OCF1B)  |				// Timer/Counter1 Output Compare B Match Flag
		//	_BV(ICF1)   |				// Timer/Counter1 Input Capture Flag
		//	_BV(OCF0B)  |				// Timer/Counter0 Output Compare Flag 0 B
		//	_BV(TOV0)   |				// Timer/Counter0 Overflow Flag
		//	_BV(OCF0A)  |				// Timer/Counter0 Output Compare Flag 0 A
			0;							//

	// USI (I2C slave mode)
	PORTB |=  _BV(PB7);					// SCL high level
	PORTB |=  _BV(PB5);					// SDA high level
	DDRB  |=  _BV(DDB7);				// SCL output
	DDRB  &= ~_BV(DDB5);				// SDA input
	USICR =								// USI Control Register
			_BV(USISIE) |				// Start Condition Interrupt Enable
			_BV(USIOIE) |				// Counter Overflow Interrupt Enable
			_BV(USIWM1) |				// Wire Mode   0  0  1  1
			_BV(USIWM0) |				//             0  1  0  1
										//             +--|--|--|-- disable
										//                +--|--|-- Three-wire mode
										//                   +--|-- Two-wire mode
										//                      +-- Two-wire mode(except that the SCL line is also held low when a counter overflow occurs, and is held low until the Timer Overflow Flag (USIOIF) is cleared)
										//
			_BV(USICS1) |				// Clock Source Select  0  0  0  1  1  1  1
		//	_BV(USICS0) |				//                      0  0  1  0  1  0  1
		//	_BV(USICLK) |				// Clock Strobe         0  1  X  0  0  1  1   Shift Register Clock Source    4-bit Counter Clock Source
										//                      +--|--|--|--|--|--|-- No Clock                       No Clock 
										//                         +--|--|--|--|--|-- Software clock strobe(USICLK)  Software clock strobe(USICLK) 
										//                            +--|--|--|--|-- Timer/Counter0 overflow        Timer/Counter0 overflow 
										//                               +--|--|--|-- External, positive edge        External, both edges 
										//                                  +--|--|-- External, negative edge        External, both edges 
										//                                     +--|-- External, positive edge        Software clock strobe (USITC) 
										//                                        +-- External, negative edge        Software clock strobe (USITC)
										//
		//	_BV(USITC)  |				// Toggle Clock Port Pin
			0;

	USISR =								// USI Status Register
			_BV(USISIF)  |				// Start Condition Interrupt Flag
			_BV(USIOIF)  |				// Counter Overflow Interrupt Flag
			_BV(USIPF)   |				// Stop Condition Flag
			_BV(USIDC)   |				// Data Output Collision
		//	_BV(USICNT3) |				// Counter Value
		//	_BV(USICNT2) |				// 
		//	_BV(USICNT1) |				// 
		//	_BV(USICNT0) |				// 
			0;

	//
	// Main loop
	//
	set_sleep_mode(SLEEP_MODE_IDLE);
	sei();
	for(;;) {
		sleep_mode();
	}
}
